% ip_response.m: Plot time response of the inverted pendulum based on the output of ip_ode.m and ip_cs.m.

clear all
close all
period = 3;
method = 2;  % <<===== to be changed manually!
options1 = odeset('RelTol', 1e-3);

[t1,y1]=ode23('ip_ode',[0 period],[70*pi/180 0 35*pi/180 0]',options1,method);
[t2,y2]=ode23('ip_ode',[0 period],[35*pi/180 0 17.5*pi/180 0]',options1,method);
[t3,y3]=ode23('ip_ode',[0 period],[-35*pi/180 0 -17.5*pi/180 0]',options1,method);
[t4,y4]=ode23('ip_ode',[0 period],[-70*pi/180 0 -35*pi/180 0]',options1,method);

% system states
figure(1);
plot(t1,y1(:,1),'k',t2,y2(:,1),'k--',t3,y3(:,1),'k-.',t4,y4(:,1),'k:');
L = legend('$\mathbf{x}(0) = [\frac{70\pi}{180}\ 0]^T$', '$\mathbf{x}(0) = [\frac{35\pi}{180}\ 0]^T$', ...
    '$\mathbf{x}(0) = [-\frac{35\pi}{180}\ 0]^T$', '$\mathbf{x}(0) = [-\frac{70\pi}{180}\ 0]^T$');
set(L,'interpreter','latex')
ylabel('System state $x_1(t)\ (rad)$','interpreter','latex');
xlabel('Time $t\ (s)$','interpreter','latex');

figure(2);
plot(t1,y1(:,2),'k',t2,y2(:,2),'k--',t3,y3(:,2),'k-.',t4,y4(:,2),'k:');
L = legend('$\mathbf{x}(0) = [\frac{70\pi}{180}\ 0]^T$', '$\mathbf{x}(0) = [\frac{35\pi}{180}\ 0]^T$', ...
    '$\mathbf{x}(0) = [-\frac{35\pi}{180}\ 0]^T$', '$\mathbf{x}(0) = [-\frac{70\pi}{180}\ 0]^T$');
set(L,'interpreter','latex')
ylabel('System state $x_2(t)\ (rad/s)$','interpreter','latex');
xlabel('Time $t\ (s)$','interpreter','latex');

figure(3);
plot(t1,y1(:,1),'k',t1,y1(:,3),'k--');
L = legend('$x_1(t)$', '$\breve{x}_1(t)$');
set(L,'interpreter','latex');
ylabel('$x_1(t)$ and $\breve{x}_1(t)\ (rad)$','interpreter','latex');
xlabel('Time $t\ (s)$','interpreter','latex');

figure(4);
plot(t1,y1(:,2),'k',t1,y1(:,4),'k--');
L = legend('$x_2(t)$', '$\breve{x}_2(t)$');
set(L,'interpreter','latex');
ylabel('$x_2(t)$ and $\breve{x}_2(t)\ (rad/s)$','interpreter','latex');
xlabel('Time $t\ (s)$','interpreter','latex');


% control input
u1=ip_cs(t1,y1,method);
figure(5);
plot(t1,u1,'k');
ylabel('Control input $u(t)\ (N)$','interpreter','latex');
xlabel('Time $t\ (s)$','interpreter','latex');


% % control input zoom in
% period = 0.1;
% [t1,y1]=ode23('ip_ode',[0 period],[75*pi/180 0 0 0]',options1,method);
% u1=ip_cs(t1,y1,method);
% figure(6);
% plot(t1,u1,'k');
% ylabel('Control input $u(t)$','interpreter','latex');
% xlabel('Time $t$ (seconds)','interpreter','latex');




