% ip_responsesp.m: Plot time response of the inverted pendulum based on the output of ip_odesp.m and ip_cssp.m.

clear all
close all
period = 10;
h = 0.05; % sampling time
method = 2;  % <<===== to be changed manually!
options1 = odeset('RelTol', 1e-3);

sol1=ddesd(@(t,y,Z)ip_odesp(t,y,Z,method),@(t,y)sampling(t,y,h),[70*pi/180 0 35*pi/180 0]',[0 period],options1);
sol2=ddesd(@(t,y,Z)ip_odesp(t,y,Z,method),@(t,y)sampling(t,y,h),[35*pi/180 0 17.5*pi/180 0]',[0 period],options1);
sol3=ddesd(@(t,y,Z)ip_odesp(t,y,Z,method),@(t,y)sampling(t,y,h),[-35*pi/180 0 -17.5*pi/180 0]',[0 period],options1);
sol4=ddesd(@(t,y,Z)ip_odesp(t,y,Z,method),@(t,y)sampling(t,y,h),[-70*pi/180 0 -35*pi/180 0]',[0 period],options1);

t1 = sol1.x;
t2 = sol2.x;
t3 = sol3.x;
t4 = sol4.x;
y1 = deval(sol1,t1);
y2 = deval(sol2,t2);
y3 = deval(sol3,t3);
y4 = deval(sol4,t4);

% system states
figure(1);
plot(t1,y1(1,:),'k',t2,y2(1,:),'k--',t3,y3(1,:),'k-.',t4,y4(1,:),'k:');
L = legend('$\mathbf{x}(0) = [\frac{70\pi}{180}\ 0]^T$', '$\mathbf{x}(0) = [\frac{35\pi}{180}\ 0]^T$', ...
    '$\mathbf{x}(0) = [-\frac{35\pi}{180}\ 0]^T$', '$\mathbf{x}(0) = [-\frac{70\pi}{180}\ 0]^T$');
set(L,'interpreter','latex')
ylabel('System state $x_1(t)\ (rad)$','interpreter','latex');
xlabel('Time $t\ (s)$','interpreter','latex');

figure(2);
plot(t1,y1(2,:),'k',t2,y2(2,:),'k--',t3,y3(2,:),'k-.',t4,y4(2,:),'k:');
L = legend('$\mathbf{x}(0) = [\frac{70\pi}{180}\ 0]^T$', '$\mathbf{x}(0) = [\frac{35\pi}{180}\ 0]^T$', ...
    '$\mathbf{x}(0) = [-\frac{35\pi}{180}\ 0]^T$', '$\mathbf{x}(0) = [-\frac{70\pi}{180}\ 0]^T$');
set(L,'interpreter','latex')
ylabel('System state $x_2(t)\ (rad/s)$','interpreter','latex');
xlabel('Time $t\ (s)$','interpreter','latex');

figure(3);
plot(t1,y1(1,:),'k',t1,y1(3,:),'k--');
L = legend('$x_1(t)$', '$\breve{x}_1(t)$');
set(L,'interpreter','latex');
ylabel('$x_1(t)$ and $\breve{x}_1(t)\ (rad)$','interpreter','latex');
xlabel('Time $t\ (s)$','interpreter','latex');

figure(4);
plot(t1,y1(2,:),'k',t1,y1(4,:),'k--');
L = legend('$x_2(t)$', '$\breve{x}_2(t)$');
set(L,'interpreter','latex');
ylabel('$x_2(t)$ and $\breve{x}_2(t)\ (rad/s)$','interpreter','latex');
xlabel('Time $t\ (s)$','interpreter','latex');


% control input
u1=ip_cssp(t1,y1,method);
figure(5);
plot(t1,u1,'k');
ylabel('Control input $u(t)\ (N)$','interpreter','latex');
xlabel('Time $t\ (s)$','interpreter','latex');


% sampled output
ss = [0:0.05:10];
xx = deval(sol1,ss);

xs1 = xx(1,:);
xs3 = xx(3,:);
xs = [xs1];
xhs = [xs3];

%===== Parameters of Inverted Pendulum =====
g=9.8;
L=0.5;
mp=2;
Mc=8;
a=1/(mp+Mc);

x1min = -70*pi/180;
x1max = 70*pi/180;

xx1=x1min:0.01:x1max;

f1min = min(cos(xx1)./(4*L/3 - a*mp*L*cos(xx1).^2));
f1max = max(cos(xx1)./(4*L/3 - a*mp*L*cos(xx1).^2));

cc = 0.1;
C(:,:,1)=1+cc;
C(:,:,2)=1-cc;

%===== Sampled Membership functions of fuzzy controller =====
f1h = cos(xs1)./(4*L/3 - a*mp*L*cos(xs1).^2);  % f1 = W1*f1min + W2*f1max
W1s = (f1h - f1max)/(f1min - f1max);
W2s = 1 - W1s;

%===== Sampled Membership functions of fuzzy controller =====
f1h = cos(xs3)./(4*L/3 - a*mp*L*cos(xs3).^2);  % f1 = W1*f1min + W2*f1max
M1s = (f1h - f1max)/(f1min - f1max);
M2s = 1 - M1s;

yy = (W1s*C(:,:,1) + W2s*C(:,:,2)).*xs;
yh = (M1s*C(:,:,1) + M2s*C(:,:,2)).*xhs;

figure(6);
stairs(ss,yy,'k');
hold on
stairs(ss,yh,'k:');
L = legend('$y_s(t)$', '$\breve{y}_s(t)$');
set(L,'interpreter','latex');
ylabel('$y_s(t)$ and $\breve{y}_s(t)$','interpreter','latex');
xlabel('Time $t\ (s)$','interpreter','latex');



% output zoom in
ss = [0:0.05:1];
xx = deval(sol1,ss);

xs1 = xx(1,:);
xs3 = xx(3,:);
xs = [xs1];
xhs = [xs3];

%===== Parameters of Inverted Pendulum =====
g=9.8;
L=0.5;
mp=2;
Mc=8;
a=1/(mp+Mc);

x1min = -70*pi/180;
x1max = 70*pi/180;

xx1=x1min:0.01:x1max;

f1min = min(cos(xx1)./(4*L/3 - a*mp*L*cos(xx1).^2));
f1max = max(cos(xx1)./(4*L/3 - a*mp*L*cos(xx1).^2));

cc = 0.1;
C(:,:,1)=1+cc;
C(:,:,2)=1-cc;

%===== Sampled Membership functions of fuzzy controller =====
f1h = cos(xs1)./(4*L/3 - a*mp*L*cos(xs1).^2);  % f1 = W1*f1min + W2*f1max
W1s = (f1h - f1max)/(f1min - f1max);
W2s = 1 - W1s;

%===== Sampled Membership functions of fuzzy controller =====
f1h = cos(xs3)./(4*L/3 - a*mp*L*cos(xs3).^2);  % f1 = W1*f1min + W2*f1max
M1s = (f1h - f1max)/(f1min - f1max);
M2s = 1 - M1s;

yy = (W1s*C(:,:,1) + W2s*C(:,:,2)).*xs;
yh = (M1s*C(:,:,1) + M2s*C(:,:,2)).*xhs;

figure(7);
stairs(ss,yy,'k');
hold on
stairs(ss,yh,'k:');
L = legend('$y_s(t)$', '$\breve{y}_s(t)$');
set(L,'interpreter','latex');
ylabel('$y_s(t)$ and $\breve{y}_s(t)$','interpreter','latex');
xlabel('Time $t\ (s)$','interpreter','latex');

% % control input zoom in
% period = 0.1;
% sol1=ddesd(@(t,y,Z)ip_odesp(t,y,Z,method),@(t,y)sampling(t,y,h),[70*pi/180 0 0 0]',[0 period],options1);
% t1 = sol1.x;
% y1 = deval(sol1,t1);
% u1=ip_cssp(t1,y1,method);
% figure(6);
% plot(t1,u1,'k');
% ylabel('Control input $u(t)$','interpreter','latex');
% xlabel('Time $t$ (seconds)','interpreter','latex');




