% msd_sos.m: Stability conditions of PFMB observer-control system for the nonlinear mass-spring-damping system.

clear all;
close all;

pdc = 1;

syms x1 x2 x3 x4 x5 x6 x7 x8 %  x5; x6; x7; x8 are estimated states of x1 x2 x3 x4, respectively.
varX=[x1; x2; x3; x4; x5; x6; x7; x8];


c1 = 0;
c2 = 1;
c3 = - 0.3;
c4 = 0.01;
c5 = 0.1;
c6 = 0.3;
c7 = - 0.03;
c8 = 0.2;
m = 1;
D = 1;
K = 1;


x1min = - pi;
x1max = pi;

xx2 = x1min:pi/50:x1max;
f1 = cos(5*xx2);

a(1) = min(f1);
a(2) = max(f1);

for i = 1:2
    
    
    A(:,:,i) = [0  1;
        - 1/m*((D*c1 + K*(c4 + c6)) + K*c5*x1^2),  - 1/m*(D*c3 + D*c2*x2^2)];
    
    Ah(:,:,i) = [0  1;
        - 1/m*((D*c1 + K*(c4 + c6)) + K*c5*x3^2),  - 1/m*(D*c3 + D*c2*x4^2)];
    
    B(:,:,i) = [0;  1/m*(1.4387 + c7*x2^2 + c8*a(i))];
    
    Bh(:,:,i) = [0;  1/m*(1.4387 + c7*x4^2 + c8*a(i))];
    
    C(:,:,i) = [1  0];
    
    Ch(:,:,i) = [1  0];
end


% Dimensions
[dimen_n, dimen_N, np]=size(A);
[trash1, dimen_m, trash2]=size(B);
[dimen_l, trash3, trash4]=size(C);

nc = np; % number of rules for controllers

% =============================================
% Initialize the sum of squares program
prog = sosprogram([varX]);

% Matrix variables
VECX = monomials(varX,[0]); % X
[prog,X] = sospolymatrixvar(prog,[VECX],[dimen_N dimen_N],'symmetric');

% Matrix variables
VECY = monomials(varX,[0]); % Y
[prog,Y] = sospolymatrixvar(prog,[VECY],[dimen_N dimen_N],'symmetric');

VECM = monomials([x3],[0:2:2]); % M
for k=1:nc
    [prog,M(:,:,k)] = sospolymatrixvar(prog,[VECM],[dimen_m dimen_N]);
end

VECN = monomials([x3],[0:2:2]); % N
for k=1:np
    [prog,N(:,:,k)] = sospolymatrixvar(prog,[VECN],[dimen_N dimen_l]);
end

% =============================================
% Predefined scalar polynomials for positive definite rather than semi-positive
varsigma1=1e-3;
varsigma2=1e-3;
varsigma3=1e-7;

alpha1 = 1e3;
alpha2 = 1e6;
beta = 1e-2;

% =============================================
%SOS conditions
expr = X - varsigma1*eye(dimen_N);
prog = sosmatrixineq(prog,expr,'Mineq');

expr = Y - varsigma2*eye(dimen_N);
prog = sosmatrixineq(prog,expr,'Mineq');

% for k=1:nc
%     expr = [1e5*eye(dimen_n) transpose(M(:,:,k)); M(:,:,k) 1];
%     prog = sosmatrixineq(prog,expr,'Mineq');
% end

Z = zeros(dimen_N,dimen_N);
I = eye(dimen_N);

switch(pdc)
    case(1) % PDC
        for i=1:np
            for j=1:np
                for k=1:np
                    if j<=k
                        
                        G11jk = Ah(:,:,j)*X - Bh(:,:,j)*M(:,:,k);
                        G11kj = Ah(:,:,k)*X - Bh(:,:,k)*M(:,:,j);
                        
                        G21jk = (A(:,:,i) - Ah(:,:,j))*X - (B(:,:,i) - Bh(:,:,j))*M(:,:,k);
                        G21kj = (A(:,:,i) - Ah(:,:,k))*X - (B(:,:,i) - Bh(:,:,k))*M(:,:,j);
                        
                        G12jk = N(:,:,j)*C(:,:,i);
                        G12kj = N(:,:,k)*C(:,:,i);
                        
                        G22jk = Y*A(:,:,i) - N(:,:,j)*C(:,:,i);
                        G22kj = Y*A(:,:,i) - N(:,:,k)*C(:,:,i);
                        
                        Hjk = N(:,:,j)*(C(:,:,i) - Ch(:,:,k));
                        Hkj = N(:,:,k)*(C(:,:,i) - Ch(:,:,j));
                        
                        Xjk = [G11jk+transpose(G11jk),  transpose(G21jk),  I,  X;
                            G21jk,   -alpha2*I,  Z,  Z;
                            I,  Z,  -1/alpha1*Y,  Z;
                            X,  Z,  Z,  -1/beta*I];
                        
                        Xkj = [G11kj+transpose(G11kj),  transpose(G21kj),  I,  X;
                            G21kj,   -alpha2*I,  Z,  Z;
                            I,  Z,  -1/alpha1*Y,  Z;
                            X,  Z,  Z,  -1/beta*I];
                        
                        Yjk = [-alpha1*Y,  G12jk,  Z,  Hjk;
                            transpose(G12jk),  G22jk+transpose(G22jk),  Y,  -Hjk;
                            Z,  Y,  -1/alpha2*I,  Z;
                            transpose(Hjk),  -transpose(Hjk),  Z,  -beta*I];
                        
                        Ykj = [-alpha1*Y,  G12kj,  Z,  Hkj;
                            transpose(G12kj),  G22kj+transpose(G22kj),  Y,  -Hkj;
                            Z,  Y,  -1/alpha2*I,  Z;
                            transpose(Hkj),  -transpose(Hkj),  Z,  -beta*I];
                        
                        expr = - Xjk - Xkj - varsigma3*eye(4*dimen_N);
                        prog = sosmatrixineq(prog,expr,'Mineq');
                        expr = - Yjk - Ykj - varsigma3*eye(4*dimen_N);
                        prog = sosmatrixineq(prog,expr,'Mineq');
                    end
                end
            end
        end
end

% =============================================
% Call solver
[prog, info] = sossolve(prog);

% Solutions
SOLX = sosgetsol(prog,X);
SOLY = sosgetsol(prog,Y);

if det(SOLX)~=0 && det(SOLY)~=0
    for k=1:nc
        F(:,:,k) = sosgetsol(prog,M(:,:,k));
        F(:,:,k) = F(:,:,k)/SOLX;
    end
    F
    for k=1:np
        LL(:,:,k) = sosgetsol(prog,N(:,:,k));
        LL(:,:,k) = inv(SOLY)*LL(:,:,k);
    end
    LL
end

% Feasibility criterion
if abs(info.feasratio - 1)<0.5 && info.pinf==0 && info.dinf==0 && info.numerr<=1
    tmin = -1;
else
    tmin = 1;
end

% [max_meig,meig]=sosOBSvf(F,L);
% max_meig




